@@ -27,6 +27,7 @@
 
 import os
 import json
+import numpy as np
 import pandas as pd
 
 from ._version import __version__
@@ -34,20 +35,69 @@ from ._version import __version__
 HERE = os.path.dirname(__file__)
 
 
+def attempt_to_set_type(df, column, numpy_type):
+    try:
+        df[column] = df[column].astype(numpy_type)
+    except ValueError:
+        pass
+
+
 def _json_table_to_df(json_table):
     table = json.loads(json_table)
-    columns = [t['name'] for t in table['schema']['fields']]
+    columns = [fields['name'] for fields in table['schema']['fields']]
+    types = [fields['type'] for fields in table['schema']['fields']]
     index = table['schema']['primaryKey'][0]
 
+    for column, a_type in zip(columns, types):
+        if column != index:
+            for row in table['data']:
+                is_empty = (
+                    row[column] is None or
+                    row[column] == "" or
+                    row[column] == np.nan
+                )
+                if is_empty:
+                    if a_type == "string":
+                        row[column] = ""
+                    if a_type == "number" or a_type == "integer":
+                        row[column] = np.nan
+                    if a_type == "boolean":
+                        row[column] = False
+
     df = pd.DataFrame(
         table['data'],
         columns=columns)
 
     df.set_index(index, inplace=True)
 
-    for column in columns:
+    for column, a_type in zip(columns, types):
         if column != index:
-            df[column] = df[column].astype('float64')
+            if a_type == "string":
+                attempt_to_set_type(df, column, np.dtype(str))
+
+            elif a_type == "number":
+                attempt_to_set_type(df, column, np.dtype(float))
+
+            elif a_type == "integer":
+                attempt_to_set_type(df, column, np.dtype(int))
+
+            elif a_type == "boolean":
+                attempt_to_set_type(df, column, np.dtype(bool))
+
+            elif a_type == "datetime":
+                attempt_to_set_type(df, column, np.datetime64)
+
+            elif a_type == "duration":
+                attempt_to_set_type(df, column, np.timedelta64)
+
+            elif a_type == "any":
+                attempt_to_set_type(df, column, 'category')
+
+            else:
+                raise ValueError(
+                    "Unexpected type, got {}"
+                    .format(a_type)
+                )
 
     return df
 
